@@ -5,9 +5,9 @@
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the GNU General Public License version 3,
+# as published by the Free Software Foundation.
+
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,6 +28,7 @@
 
 from logging import getLogger
 
+from plainbox.i18n import gettext as _
 from plainbox.impl.commands import PlainBoxCommand
 from plainbox.impl.commands.checkbox import CheckBoxCommandMixIn
 from plainbox.impl.commands.checkbox import CheckBoxInvocationMixIn
@@ -38,8 +39,8 @@ logger = getLogger("plainbox.commands.special")
 
 class SpecialInvocation(CheckBoxInvocationMixIn):
 
-    def __init__(self, checkbox, ns):
-        super(SpecialInvocation, self).__init__(checkbox)
+    def __init__(self, provider_list, config, ns):
+        super().__init__(provider_list, config)
         self.ns = ns
 
     def run(self):
@@ -72,12 +73,12 @@ class SpecialInvocation(CheckBoxInvocationMixIn):
     def _print_job_list(self, ns, job_list):
         matching_job_list = self._get_matching_job_list(ns, job_list)
         for job in matching_job_list:
-            print("{}".format(job))
+            print("{}".format(job.id))
 
     def _print_job_hash_list(self, ns, job_list):
         matching_job_list = self._get_matching_job_list(ns, job_list)
         for job in matching_job_list:
-            print("{} {}".format(job.get_checksum(), job))
+            print("{} {}".format(job.checksum, job.id))
 
     def _print_expression_list(self, ns, job_list):
         matching_job_list = self._get_matching_job_list(ns, job_list)
@@ -96,25 +97,25 @@ class SpecialInvocation(CheckBoxInvocationMixIn):
         print('\tnode [shape=box];')
         for job in matching_job_list:
             if job.plugin == "resource":
-                print('\t"{}" [shape=ellipse,color=blue];'.format(job.name))
+                print('\t"{}" [shape=ellipse,color=blue];'.format(job.id))
             elif job.plugin == "attachment":
-                print('\t"{}" [color=green];'.format(job.name))
+                print('\t"{}" [color=green];'.format(job.id))
             elif job.plugin == "local":
                 print('\t"{}" [shape=invtriangle,color=red];'.format(
-                    job.name))
+                    job.id))
             elif job.plugin == "shell":
-                print('\t"{}" [];'.format(job.name))
+                print('\t"{}" [];'.format(job.id))
             elif job.plugin in ("manual", "user-verify", "user-interact"):
-                print('\t"{}" [color=orange];'.format(job.name))
-            for dep_name in job.get_direct_dependencies():
-                print('\t"{}" -> "{}";'.format(job.name, dep_name))
+                print('\t"{}" [color=orange];'.format(job.id))
+            for dep_id in job.get_direct_dependencies():
+                print('\t"{}" -> "{}";'.format(job.id, dep_id))
             prog = job.get_resource_program()
             if ns.dot_resources and prog is not None:
                 for expression in prog.expression_list:
                     print('\t"{}" [shape=ellipse,color=blue];'.format(
-                        expression.resource_name))
+                        expression.resource_id))
                     print('\t"{}" -> "{}" [style=dashed, label="{}"];'.format(
-                        job.name, expression.resource_name,
+                        job.id, expression.resource_id,
                         expression.text.replace('"', "'")))
         print("}")
 
@@ -124,36 +125,38 @@ class SpecialCommand(PlainBoxCommand, CheckBoxCommandMixIn):
     Implementation of ``$ plainbox special``
     """
 
-    def __init__(self, checkbox):
-        self.checkbox = checkbox
+    def __init__(self, provider_list, config):
+        self.provider_list = provider_list
+        self.config = config
 
     def invoked(self, ns):
-        return SpecialInvocation(self.checkbox, ns).run()
+        return SpecialInvocation(self.provider_list, self.config, ns).run()
 
     def register_parser(self, subparsers):
         parser = subparsers.add_parser(
-            "special", help="special/internal commands")
+            "special", help=_("special/internal commands"))
         parser.set_defaults(command=self)
         group = parser.add_mutually_exclusive_group(required=True)
         group.add_argument(
             '-j', '--list-jobs',
-            help="List jobs instead of running them",
+            help=_("list jobs instead of running them"),
             action="store_const", const="list-jobs", dest="special")
         group.add_argument(
             '-J', '--list-job-hashes',
-            help="List jobs with hashes instead of running them",
+            help=_("list jobs with cheksums instead of running them"),
             action="store_const", const="list-job-hashes", dest="special")
         group.add_argument(
             '-e', '--list-expressions',
-            help="List all unique resource expressions",
+            help=_("list all unique resource expressions"),
             action="store_const", const="list-expr", dest="special")
         group.add_argument(
             '-d', '--dot',
-            help="Print a graph of jobs instead of running them",
+            help=_("print a graph of jobs instead of running them"),
             action="store_const", const="dep-graph", dest="special")
         parser.add_argument(
             '--dot-resources',
-            help="Render resource relationships (for --dot)",
+            # TRANSLATORS: please keep --dot untranslated
+            help=_("show resource relationships (for --dot)"),
             action='store_true')
         # Call enhance_parser from CheckBoxCommandMixIn
         self.enhance_parser(parser)
