@@ -50,6 +50,7 @@ class Problem(SymbolDef):
     missing = 'missing'
     wrong = 'wrong'
     useless = 'useless'
+    deprecated = 'deprecated'
 
 
 class ValidationError(ValueError):
@@ -75,15 +76,27 @@ class CheckBoxJobValidator:
     """
 
     @staticmethod
-    def validate(job):
+    def validate(job, strict=False, deprecated=False):
         """
         Validate the specified job
+
+        :param strict:
+            Enforce strict validation. Non-conforming jobs will be rejected.
+            This is off by default to ensure that non-critical errors don't
+            prevent jobs from running.
+        :param deprecated:
+            Enforce deprecation validation. Jobs having deprecated fields will
+            be rejected. This is off by default to allow backwards compatible
+            jobs to be used without any changes.
         """
-        # Check if id is empty
-        if job.id is None:
+        # Check if name is still being used, if running in strict mode
+        if deprecated and job.name is not None:
+            raise ValidationError(job.fields.name, Problem.deprecated)
+        # Check if the partial_id field is empty
+        if job.partial_id is None:
             raise ValidationError(job.fields.id, Problem.missing)
-        # Check if summary is empty
-        if job.summary is None:
+        # Check if summary is empty, if running in strict mode
+        if strict and job.summary is None:
             raise ValidationError(job.fields.summary, Problem.missing)
         # Check if plugin is empty
         if job.plugin is None:
@@ -91,11 +104,13 @@ class CheckBoxJobValidator:
         # Check if plugin has a good value
         if job.plugin not in JobDefinition.plugin.get_all_symbols():
             raise ValidationError(job.fields.plugin, Problem.wrong)
-        # Check if user is given without a command to run
-        if job.user is not None and job.command is None:
+        # Check if user is given without a command to run, if running in strict
+        # mode
+        if strict and job.user is not None and job.command is None:
             raise ValidationError(job.fields.user, Problem.useless)
-        # Check if environ is given without a command to run
-        if job.environ is not None and job.command is None:
+        # Check if environ is given without a command to run, if running in
+        # strict mode
+        if strict and job.environ is not None and job.command is None:
             raise ValidationError(job.fields.environ, Problem.useless)
         # Verify that command is present on a job within the subset that should
         # really have them (shell, local, resource, attachment, user-verify and
@@ -121,8 +136,9 @@ class CheckBoxJobValidator:
             if job.description is None:
                 raise ValidationError(
                     job.fields.description, Problem.missing)
-            # Ensure that manual jobs don't have command
-            if job.command is not None:
+            # Ensure that manual jobs don't have command, if running in strict
+            # mode
+            if strict and job.command is not None:
                 raise ValidationError(job.fields.command, Problem.useless)
 
 
@@ -261,7 +277,7 @@ class JobDefinition(BaseJob, IJobDefinition):
         Get the translated version of :meth:`summary`
         """
         return self.get_normalized_translated_data(
-            self.get_raw_record_value('summary'))
+            self.get_raw_record_value('summary')) or self.id
 
     @property
     def name(self):
@@ -481,16 +497,12 @@ class JobDefinition(BaseJob, IJobDefinition):
     @classmethod
     def from_rfc822_record(cls, record):
         """
-        Create a JobDefinition instance from rfc822 record
+        Create a JobDefinition instance from rfc822 record. The resulting
+        instance may not be valid but will always be created. Only valid jobs
+        should be executed.
 
         The record must be a RFC822Record instance.
-
-        Only the 'id' and 'plugin' keys are required.
-        All other data is stored as is and is entirely optional.
         """
-        if 'id' not in record.data and 'name' not in record.data:
-            # TRANSLATORS: don't translate id or translate it as 'id field'
-            raise ValueError(_("Cannot create job without an id"))
         # Strip the trailing newlines form all the raw values coming from the
         # RFC822 parser. We don't need them and they don't match gettext keys
         # (xgettext strips out those newlines)
@@ -498,14 +510,17 @@ class JobDefinition(BaseJob, IJobDefinition):
             key: value.rstrip('\n')
             for key, value in record.raw_data.items()})
 
-    def validate(self, validator_cls=CheckBoxJobValidator):
+    def validate(self, **validation_kwargs):
         """
         Validate this job definition with the specified validator
 
+        :param validation_kwargs:
+            Keyword arguments to pass to the
+            :meth:`CheckBoxJobValidator.validate()`
         :raises ValidationError:
             If the job has any problems that make it unsuitable for execution.
         """
-        validator_cls.validate(self)
+        CheckBoxJobValidator.validate(self, **validation_kwargs)
 
     def create_child_job_from_record(self, record):
         """
