@@ -1,6 +1,6 @@
 # This file is part of Checkbox.
 #
-# Copyright 2013 Canonical Ltd.
+# Copyright 2013, 2014 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
@@ -32,6 +32,8 @@ import configparser
 import logging
 import re
 
+from plainbox.i18n import gettext as _
+
 
 logger = logging.getLogger("plainbox.config")
 
@@ -63,15 +65,13 @@ class ConfigMetaData:
 
     This class has typically two attributes:
 
-        :cvar variable_list:
-            A list of all Variable objects defined in the class
-
-        :cvar section_list:
-            A list of all Section object defined in the class
-
-        :cvar filename_list:
-            A list of config files (pathnames) to read on call to
-            :meth:`Config.read`
+    :attr variable_list:
+        A list of all Variable objects defined in the class
+    :attr section_list:
+        A list of all Section object defined in the class
+    :attr filename_list:
+        A list of config files (pathnames) to read on call to
+        :meth:`Config.read`
     """
     variable_list = []
     section_list = []
@@ -84,15 +84,32 @@ class UnsetType:
     """
 
     def __str__(self):
-        return "unset"
+        return _("unset")
 
     def __repr__(self):
         return "Unset"
 
+    def __bool__(self):
+        return False
+
 
 Unset = UnsetType()
 
 
+def understands_Unset(cls_or_func):
+    """
+    Decorator for marking validators as supporting the special Unset value.
+
+    This decorator should be applied to every validator that natively supports
+    Unset values. Without it, Unset is never validated.
+
+    This decorator works by setting the ``understands_Unset`` attribute on the
+    decorated object and returning it intact.
+    """
+    cls_or_func.understands_Unset = True
+    return cls_or_func
+
+
 class Variable(INameTracking):
     """
     Variable that can be used in a configuration systems
@@ -104,12 +121,24 @@ class Variable(INameTracking):
                  default=Unset, validator_list=None, help_text=None):
         # Ensure kind is correct
         if kind not in self._KIND_CHOICE:
-            raise ValueError("unsupported kind")
+            raise ValueError(_("unsupported kind"))
         # Ensure that we have a validator_list, even if empty
         if validator_list is None:
             validator_list = []
-        # Insert a KindValidator as the first validator to run
-        validator_list.insert(0, KindValidator)
+        if validator_list and isinstance(validator_list[0], NotUnsetValidator):
+            # XXX: Kludge ahead, beware!
+            # Insert a KindValidator as the second validator to run
+            # just after the NotUnsetValidator
+            # TODO: To properly handle this without any special-casing we
+            # should drop the implicit insertion of the KindValidator and
+            # convert all users to properly order KindValidator and
+            # NotUnsetValidator instances so that the error message is helpful
+            # to the user. The whole idea is to validate Unset before we try to
+            # validate the type.
+            validator_list.insert(1, KindValidator)
+        else:
+            # Insert a KindValidator as the first validator to run
+            validator_list.insert(0, KindValidator)
         # Assign all the attributes
         self._name = name
         self._section = section
@@ -117,20 +146,31 @@ class Variable(INameTracking):
         self._default = default
         self._validator_list = validator_list
         self._help_text = help_text
-        self._validate_default_value()
         # Workaround for Sphinx breaking if __doc__ is a property
         self.__doc__ = self.help_text or self.__class__.__doc__
 
-    def _validate_default_value(self):
+    def validate(self, value):
         """
-        Validate the default value, unless it is Unset
+        Check if the supplied value is valid for this variable.
+
+        :param value:
+            The proposed value
+        :raises ValidationError:
+            Tf the value was not valid in any way
         """
-        if self.default is Unset:
-            return
         for validator in self.validator_list:
-            message = validator(self, self.default)
+            # Most validators don't want to deal with the unset type so let's
+            # special case that.  Anything that is decorated with
+            # @understands_Unset will have that attribute set to True.
+            #
+            # If the value _is_ unset and the validator doesn't claim to
+            # support it then just skip it.
+            if value is Unset and not getattr(validator, 'understands_Unset',
+                                              False):
+                continue
+            message = validator(self, value)
             if message is not None:
-                raise ValidationError(self, self.default, message)
+                raise ValidationError(self, value, message)
 
     def _set_tracked_name(self, name):
         """
@@ -205,10 +245,7 @@ class Variable(INameTracking):
         :raises ValidationError: if the new value is incorrect
         """
         # Check it against all validators
-        for validator in self.validator_list:
-            message = validator(self, new_value)
-            if message is not None:
-                raise ValidationError(self, new_value, message)
+        self.validate(new_value)
         # Assign it to the backing store of the instance
         instance._set_variable(self.name, new_value)
 
@@ -232,7 +269,7 @@ class Section(INameTracking):
 
     def _set_tracked_name(self, name):
         """
-        Internal method used by :meth:`ConfigMeta.__new__`
+        Internal method used by :meth:`ConfigMeta.__new__()`
         """
         if self._name is None:
             self._name = name
@@ -288,13 +325,13 @@ class ConfigMeta(type):
                 section_list = namespace['Meta'].section_list[:]
         # Discover all Variable and Section instances
         # defined in the class namespace
-        for name, item in namespace.items():
-            if isinstance(item, INameTracking):
-                item._set_tracked_name(name)
-            if isinstance(item, Variable):
-                variable_list.append(item)
-            elif isinstance(item, Section):
-                section_list.append(item)
+        for attr_name, attr_value in namespace.items():
+            if isinstance(attr_value, INameTracking):
+                attr_value._set_tracked_name(attr_name)
+            if isinstance(attr_value, Variable):
+                variable_list.append(attr_value)
+            elif isinstance(attr_value, Section):
+                section_list.append(attr_value)
         # Get or create the class of the 'Meta' object.
         #
         # This class should always inherit from ConfigMetaData and whatever the
@@ -323,28 +360,52 @@ class ConfigMeta(type):
 
 class PlainBoxConfigParser(configparser.ConfigParser):
     """
-    A simple ConfigParser subclass that does not lowercase
-    key names.
+    A subclass of ConfigParser with the following changes:
+
+    - option names are not lower-cased
+    - write() has deterministic ordering (sorted by name)
     """
+
     def optionxform(self, option):
+        """
+        Overridden method from :class:`configparser.ConfigParser`.
+
+        Returns `option` without any transformations
+        """
         return option
 
+    def write(self, fp, space_around_delimiters=True):
+        """
+        Write an .ini-format representation of the configuration state.
+
+        If `space_around_delimiters` is True (the default), delimiters between
+        keys and values are surrounded by spaces. The ordering of section and
+        values within is deterministic.
+        """
+        if space_around_delimiters:
+            d = " {} ".format(self._delimiters[0])
+        else:
+            d = self._delimiters[0]
+        if self._defaults:
+            self._write_section(
+                fp, self.default_section, sorted(self._defaults.items()), d)
+        for section in self._sections:
+            self._write_section(
+                fp, section, sorted(self._sections[section].items()), d)
+
 
 class Config(metaclass=ConfigMeta):
     """
     Base class for configuration systems
 
-    :ivar _var:
+    :attr _var:
         storage backend for Variable definitions
-
-    :ivar _section:
+    :attr _section:
         storage backend for Section definitions
-
-    :ivar _filename_list:
+    :attr _filename_list:
         list of pathnames to files that were loaded by the last call to
         :meth:`read()`
-
-    :ivar _problem_list:
+    :attr _problem_list:
         list of :class:`ValidationError` that were detected by the last call to
         :meth:`read()`
     """
@@ -388,10 +449,44 @@ class Config(metaclass=ConfigMeta):
         self.read(cls.Meta.filename_list)
         return self
 
+    def get_parser_obj(self):
+        """
+        Get a ConfigParser-like object with the same data.
+
+        :returns:
+            A :class:`PlainBoxConfigParser` object with all of the data copied
+            from this :class:`Config` object.
+
+        Since :class:`PlainBoxConfigParser` is a subclass of
+        :class:`configparser.ConfigParser` it has a number of useful utility
+        methods.  By using this function one can obtain a ConfigParser-like
+        object and work with it directly.
+        """
+        parser = PlainBoxConfigParser()
+        # Write all variables that we know about
+        for variable in self.Meta.variable_list:
+            if (not parser.has_section(variable.section)
+                    and variable.section != "DEFAULT"):
+                parser.add_section(variable.section)
+            value = variable.__get__(self, self.__class__)
+            # Except Unset, we don't want that to convert to 'unset'
+            if value is not Unset:
+                parser.set(variable.section, variable.name, str(value))
+        # Write all sections that we know about
+        for section in self.Meta.section_list:
+            if not parser.has_section(section.name):
+                parser.add_section(section.name)
+            for name, value in section.__get__(self, self.__class__).items():
+                parser.set(section.name, name, str(value))
+        return parser
+
     def read_string(self, string):
         """
         Load settings from a string.
 
+        :param string:
+            The full text of INI-like configuration to parse and apply
+
         This method parses the string as an INI file using
         :class:`PlainBoxConfigParser` (a simple ConfigParser subclass that
         respects the case of key names).
@@ -410,8 +505,8 @@ class Config(metaclass=ConfigMeta):
         either Variable or Section class) is silently ignored.
 
         .. note::
-            This method resets :ivar:`_problem_list` and
-            :ivar:`_filename_list`.
+            This method resets :attr:`_problem_list`
+            and :attr:`_filename_list`.
         """
         parser = PlainBoxConfigParser()
         # Reset filename list and problem list
@@ -424,6 +519,18 @@ class Config(metaclass=ConfigMeta):
             self._problem_list.append(exc)
         self._read_commit(parser)
 
+    def write(self, stream):
+        """
+        Write configuration data to a stream.
+
+        :param stream:
+            a file-like object that can be written to.
+
+        This method recreates the content of all the configuration variables in
+        a manner that can be subsequently read back.
+        """
+        self.get_parser_obj().write(stream)
+
     def read(self, filename_list):
         """
         Load and merge settings from many files.
@@ -448,8 +555,9 @@ class Config(metaclass=ConfigMeta):
         either Variable or Section class) is silently ignored.
 
         .. note::
-            This method resets :ivar:`_problem_list` and
-            :ivar:`_filename_list`.
+
+            This method resets :attr:`_problem_list`
+            and :attr:`_filename_list`.
         """
         parser = PlainBoxConfigParser()
         # Reset filename list and problem list
@@ -457,7 +565,7 @@ class Config(metaclass=ConfigMeta):
         self._problem_list = []
         # Try loading all of the config files
         try:
-            logger.info("Loading configuration from %s", filename_list)
+            logger.info(_("Loading configuration from %s"), filename_list)
             self._filename_list = parser.read(filename_list)
         except configparser.Error as exc:
             self._problem_list.append(exc)
@@ -477,10 +585,8 @@ class Config(metaclass=ConfigMeta):
             try:
                 value = reader_fn[variable.kind](
                     variable.section, variable.name)
-            except configparser.NoSectionError:
-                continue
-            except configparser.NoOptionError:
-                continue
+            except (configparser.NoSectionError, configparser.NoOptionError):
+                value = variable.default
             # Try to assign it
             try:
                 variable.__set__(self, value)
@@ -495,6 +601,9 @@ class Config(metaclass=ConfigMeta):
                 continue
             # Assign it
             section.__set__(self, value)
+        # Validate the whole configuration object
+        self.validate_whole()
+
 
     def _get_variable(self, name):
         """
@@ -532,6 +641,18 @@ class Config(metaclass=ConfigMeta):
         """
         del self._section[name]
 
+    def validate_whole(self):
+        """
+        Validate the whole configuration object.
+
+        This method may be overridden to provide whole-configuration
+        validation. It is especially useful in cases when a pair or more of
+        variables need to be validated together to be meaningful.
+
+        The default implementation does nothing. Other implementations may
+        raise :class:`ValidationError`.
+        """
+
 
 class ValidationError(ValueError):
     """
@@ -567,7 +688,12 @@ def KindValidator(variable, new_value):
     A validator ensuring that values match the "kind" of the variable.
     """
     if not isinstance(new_value, variable.kind):
-        return "expected a {}".format(variable.kind.__name__)
+        return {
+            bool: _("expected a boolean"),
+            int: _("expected an integer"),
+            float: _("expected a floating point number"),
+            str: _("expected a string"),
+        }[variable.kind]
 
 
 class PatternValidator(IValidator):
@@ -581,7 +707,13 @@ class PatternValidator(IValidator):
 
     def __call__(self, variable, new_value):
         if not self.pattern.match(new_value):
-            return "does not match pattern: {!r}".format(self.pattern_text)
+            return _("does not match pattern: {!r}").format(self.pattern_text)
+
+    def __eq__(self, other):
+        if isinstance(other, PatternValidator):
+            return self.pattern_text == other.pattern_text
+        else:
+            return False
 
 
 class ChoiceValidator(IValidator):
@@ -594,7 +726,42 @@ class ChoiceValidator(IValidator):
 
     def __call__(self, variable, new_value):
         if new_value not in self.choice_list:
-            return "must be one of {}".format(", ".join(self.choice_list))
+            return _("must be one of {}").format(", ".join(self.choice_list))
+
+    def __eq__(self, other):
+        if isinstance(other, ChoiceValidator):
+            return self.choice_list == other.choice_list
+        else:
+            return False
+
+
+@understands_Unset
+class NotUnsetValidator(IValidator):
+    """
+    A validator ensuring that values are set
+
+    ..note::
+        Due to the way validators work this validator *must* be the first
+        one in any validator list in order to work. Otherwise the implicit
+        :func:`KindValidator` will take precedence and the check will most
+        likely fail as None or Unset are not of the expected type of the
+        configuration variable being worked with.
+    """
+
+    def __init__(self, msg=None):
+        if msg is None:
+            msg = _("must be set to something")
+        self.msg = msg
+
+    def __call__(self, variable, new_value):
+        if new_value is Unset:
+            return self.msg
+
+    def __eq__(self, other):
+        if isinstance(other, NotUnsetValidator):
+            return self.msg == other.msg
+        else:
+            return False
 
 
 class NotEmptyValidator(IValidator):
@@ -604,9 +771,15 @@ class NotEmptyValidator(IValidator):
 
     def __init__(self, msg=None):
         if msg is None:
-            msg = "cannot be empty"
+            msg = _("cannot be empty")
         self.msg = msg
 
     def __call__(self, variable, new_value):
         if new_value == "":
             return self.msg
+
+    def __eq__(self, other):
+        if isinstance(other, NotEmptyValidator):
+            return self.msg == other.msg
+        else:
+            return False
